<?php

class Smartcat_Integration_Hub_Admin
{

    private $plugin_name;
    private $version;
    private $logger;
    private $wpdb;

    public function __construct($plugin_name, $version, LoggerInterface $logger = null)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->logger = $logger;

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    public function enqueue_styles()
    {
        // none
    }

    public function enqueue_scripts()
    {
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/smartcat-wpml-admin.js', array('jquery'),
            $this->version, true);

    }

    public function add_plugin_admin_menu()
    {
        add_options_page(
            'Integration Manager',
            'Smartcat Integration for WPML',
            'manage_options',
            $this->plugin_name,
            [$this, 'display_plugin_setup_page']
        );
    }

    public function add_action_links($links)
    {
        $settings_link = [
            '<a href="' . admin_url('options-general.php?page=' . $this->plugin_name) . '">' . __('Settings', $this->plugin_name) . '</a>'
        ];
        return array_merge($settings_link, $links);
    }

    public function display_plugin_setup_page()
    {

        if ($_SERVER["REQUEST_METHOD"] === "POST") {
            if (isset($_POST["_type"])) {
                switch ($_POST["_type"]) {
                    case "security":
                        $this->updateSecurity($_POST);
                        break;
                }
            }
        }

        include_once('partials/smartcat-wpml-display.php');

        unset($_POST["smartcat_message"]);
        unset($_POST["smartcat_message_type"]);
    }

    public function smartcatUpdateOptions()
    {
        $smartcat_update_options_nonce = sanitize_text_field($_POST["smartcat_update_options_nonce"]);
        if (isset($smartcat_update_options_nonce) && wp_verify_nonce($smartcat_update_options_nonce, 'smartcat_update_options_form_nonce')) {

            $smartcat_notice_status = "success";
            $smartcat_notice_content = "Settings have been saved";

            $secret = sanitize_text_field($_POST["secret"]);
            $acf = sanitize_text_field($_POST["acf"]);

            if (
                isset($secret) &&
                is_string($secret) &&
                !empty($secret) &&
                iconv_strlen($secret) === 30
            ) {
                update_option('smartcat_wpml_secret', $secret);
            } else {
                $smartcat_notice_status = "error";
                $smartcat_notice_content = "Incorrect API key value";
            }

            if ($acf === "on") {
                if (!class_exists('ACF')) {
                    $smartcat_notice_status = "error";
                    $smartcat_notice_content = "You cannot enable the \"Advanced Custom Fields\" option because the ACF plugin is not installed or activated";
                } else {
                    update_option('smartcat_wpml_acf_using', 1);
                }


            } else {
                update_option('smartcat_wpml_acf_using', 0);
            }

            wp_redirect(
                esc_url_raw(
                    add_query_arg(["smartcat_notice_status" => $smartcat_notice_status, "smartcat_notice_content" => $smartcat_notice_content], admin_url("options-general.php?page={$this->plugin_name}"))
                )
            );
            die();
        } else {
            wp_die(__('Invalid nonce specified', $this->plugin_name), __('Error', $this->plugin_name), [
                'response' => 403,
                'back_link' => 'admin.php?page=' . $this->plugin_name,
            ]);
        }
    }

    public function printPluginAdminNotices()
    {
        $smartcat_notice_status = sanitize_text_field($_REQUEST['smartcat_notice_status'] ?? NULL);
        if (!empty($smartcat_notice_status)) {
            $smartcat_notice_content = sanitize_text_field($_REQUEST['smartcat_notice_content']);
            ?>
            <div class="notice notice-<?php echo esc_html__($smartcat_notice_status) ?>">
                <p>
                    <?php echo esc_html__($smartcat_notice_content) ?>
                </p>
            </div>
            <?php
        } else {
            return;
        }
    }

    public function notExistWpmlNotice()
    {
        ?>
        <div class="error notice">
            <p><b>Smartcat Integration
                    Add-on:</b> <?php _e('WPML plugin is not installed or activated. Smartcat Integration Add-on is not active', 'smartcat-wpml'); ?>
            </p>
        </div>
        <?php
    }

    public function notFriendlyUrlNotice()
    {
        ?>
        <div class="error notice">
            <p><b>Smartcat Integration
                    Add-on:</b> <?php _e("The site uses simple links without a Friendly URL. Activate Friendly URL for Smartcat Integration Add-on to work correctly. <a href='https://wordpress.org/support/article/using-permalinks/' target='_blank'>More</a>", 'smartcat-wpml'); ?>
            </p>
        </div>
        <?php
    }

}
