<?php

class Smartcart_Acf_Helper
{

    const ALLOWED_TYPES = ["text", "textarea", "wysiwyg", "repeater", "flexible_content", "group"];
    const MORE_FIELDS_TYPES = ["repeater", "flexible_content"];
    const GROUP_TYPE = "group";

    private $wpdb;
    private $log;

    public function __construct()
    {
        $this->log = new Smartcart_Logs();
        global $wpdb;
        $this->wpdb = $wpdb;
    }

    public function customFields($postId): array
    {
        $fields = $this->filter((array)get_field_objects($postId));
        $fields = array_values(array_map(function ($field) use ($postId) {
            $value = [];
            if ($field["type"] === "flexible_content") {
                foreach (get_field($field["key"], $postId) as $k => $val) {
                    foreach ($val as $u => $v) {
                        if ($u !== "acf_fc_layout") {
                            $value[$k][$u] = $v;
                        }
                    }
                }
            } else {
                $value = get_field($field["key"], $postId);
            }
            return [
                "key" => $field["key"],
                "type" => $field["type"],
                "name" => $field["name"],
                "content" => $value
            ];
        }, $fields));

        foreach ($fields as $key => $field) {
           if (in_array($field["type"], self::MORE_FIELDS_TYPES)) {
               foreach ($field["content"] as $k => $value) {
                   foreach ($value as $u => $val) {
                       if (!is_string($val)) {
                           unset($fields[$key]["content"][$k][$u]);
                       }
                   }
               }
           } elseif ($field["type"] === self::GROUP_TYPE) {
               foreach ($field["content"] as $k => $value) {
                   if (!is_string($value)) {
                       unset($fields[$key]["content"][$k]);
                   }
               }
           }
        }

        return $fields;
    }

    public function filter($fields)
    {
        return array_filter($fields, function ($field) {
            return in_array($field["type"], self::ALLOWED_TYPES);
        });
    }
    
}