<?php

class Smartcart_All_Posts
{

    private $wpdb;
    private $disableSercretVerificaton = false;

    private $helpers;
    private $log;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     * @throws Exception
     */
    public function request($request)
    {
        $this->validateRequest($request);
        $params = $request->get_params();

        $prepareQuery = $this->wpdb->prepare("select * from {$this->wpdb->prefix}icl_translations where (element_type = %s or element_type = %s) and language_code = %s", ['post_post', 'post_page', $params['lang']]);
        $trids = $this->wpdb->get_results($prepareQuery);
        $posts = [];
        foreach ($trids as $trid) {
            $post = get_post($trid->element_id);
            if (is_null($post)) {
                continue;
            }
            $translations = apply_filters( 'wpml_get_element_translations', NULL, $trid->trid);
            $languages = [];
            foreach ($translations as $translation) {
                if ($translation->language_code !== $trid->language_code) {
                    $languages[] = $translation->language_code;
                }
            }
            $posts[] = [
                'id' => (int)$trid->element_id,
                'type' => $trid->element_type,
                'trid' => (int)$trid->trid,
                'title' => $post->post_title,
                'languages' => $languages
            ];
        }
        return [
            "posts" => $posts,
        ];
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest($request)
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }

        if (!$this->helpers->verifySecretKey($request) && $this->disableSercretVerificaton === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }
    }
}