<?php

class Smartcart_Get_Posts
{

    private $wpdb;
    private $helpers;
    private $log;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     * @throws Exception
     */
    public function request()
    {
        $this->validateRequest();
        return $this->normalizePosts(
            (array)$this->fetchPosts()
        );
    }

    public function where()
    {
        return array_merge(get_posts(), get_pages());
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest()
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }
    }

    private function fetchPosts()
    {
        $query = "SELECT * FROM {$this->wpdb->prefix}icl_translations WHERE element_type = %s OR element_type = %s";
        $args = ["post_post", "post_page"];
        return $this->wpdb->get_results(
            $this->wpdb->prepare($query, $args)
        );
    }

    private function normalizePosts($posts)
    {
        return array_map(function ($post) {
            return [
                "id" => (int)$post->element_id,
                "locale" => $post->language_code,
                "groupId" => (int)$post->trid
            ];
        }, $posts);
    }
}