<?php

class Smartcart_Languages
{

    private $wpdb;
    private $disableSercretVerificaton = false;

    private $helpers;
    private $log;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     * @throws Exception
     */
    public function request($request)
    {
        $this->validateRequest($request);

        $normalizeLanguages = [];
        foreach (apply_filters('wpml_active_languages', NULL, 'orderby=id&order=desc') as $language) {
            $arr = ["locale" => $language["code"]];
            if ((int)$language['active'] === 1) {
                $arr["isDefault"] = true;
            }
            $normalizeLanguages[] = $arr;
        }

        return [
            "locales" => $normalizeLanguages
        ];
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest($request)
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }

        if (!$this->helpers->verifySecretKey($request) && $this->disableSercretVerificaton === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }
    }
}