<?php

class Smartcart_Logs
{

    private $wpdb;
    private $table = "smartcat_logs";
    private $disableSercretVerification = true;

    public function __construct()
    {
        global $wpdb;
        $this->wpdb = $wpdb;
    }

    public function add(string $type = "Error", string $message = "", string $exception = "")
    {
        $this->wpdb->insert($this->wpdb->prefix . $this->table, [
            "type" => $type,
            "message" => $message,
            "exception" => $exception
        ]);
    }

    public function request($request)
    {
        $this->validateRequest($request);
        return $this->fetchLogs();
    }

    private function fetchLogs()
    {
        return $this->wpdb->get_results("SELECT * FROM {$this->wpdb->prefix}{$this->table}");
    }

    private function validateRequest($request)
    {
        if (!$this->verifySecretKey($request) && $this->disableSercretVerification === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }
    }

    private function verifySecretKey($request)
    {
        $requestData = $request->get_params();
        $secret = $request->get_header("X-Smartcat-Secret");
        $verifyHash = hash_hmac('sha256', json_encode($requestData), get_option('smartcat_wpml_secret'));

        return $secret === $verifyHash;
    }
}