<?php

class Smartcart_Pull_Categories
{

    private $wpdb;
    private $disableSercretVerificaton = false;

    private $request;
    private $helpers;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     * @throws Exception
     */
    public function request($request)
    {
        $this->request = $request;
        $this->validateRequest();

        return [
            "data" => [
                "status" => "success",
                "documents" => $this->getCategories()
            ]
        ];
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest()
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }

        if (!$this->helpers->verifySecretKey($this->request) && $this->disableSercretVerificaton === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }

        if (is_null($this->requestBody($this->request)->sourceLang)) {
            throw new ErrorException("Source lang not found", 500);
        }
    }

    private function requestBody($request)
    {
        $requestData = $request->get_params();
        return (object)[
            "sourceLang" => $requestData["sourceLang"] ?? NULL
        ];
    }

    private function getCategories(): array
    {
        return $this->fetchCategories(
            $this->fetchCategoriesId()
        );
    }

    private function fetchCategoriesId(): array
    {
        $sourceLang = $this->requestBody($this->request)->sourceLang;

        $query = "SELECT element_id FROM {$this->wpdb->prefix}icl_translations WHERE element_type = %s AND language_code = %s";
        $args = ["tax_category", $sourceLang];

        $categoriesIds = $this->wpdb->get_results(
            $this->wpdb->prepare($query, $args)
        );

        return array_map(function ($item) {
            return (int)$item->element_id;
        }, $categoriesIds);
    }

    private function fetchCategories(array $ids): array
    {

        $categoriesList = [];

        foreach ($ids as $id) {
            $category = get_category($id);
            if (is_null($category)) {
                continue;
            }
            $categoriesList[] = [
                "id" => $category->term_id,
                "title" => $category->name,
                "description" => $category->description,
                "parent" => $category->parent
            ];
        }

        return $categoriesList;
    }
}