<?php

class Smartcart_Push_Categories
{

    private $wpdb;
    private $disableSercretVerificaton = false;

    private $elementType = "tax_category";
    private $request;
    private $helpers;
    private $log;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     * @throws Exception
     */
    public function request($request)
    {
        $this->request = $request;
        $this->validateRequest();

        $categories = $this->requestBody()->categories;

        foreach ($categories as $category) {
            $this->insertCategory($category);
        }

        return [
            "data" => [
                "status" => "success"
            ]
        ];
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest()
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }

        if (!$this->helpers->verifySecretKey($this->request) && $this->disableSercretVerificaton === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }

        if (is_null($this->requestBody($this->request)->categories) || empty($this->requestBody($this->request)->categories)) {
            throw new ErrorException("Categories list is empty", 500);
        }
    }

    private function requestBody()
    {
        $requestData = $this->request->get_params();
        return (object)[
            "sourceLang" => $requestData["sourceLang"] ?? NULL,
            "categories" => $requestData["categories"] ?? NULL
        ];
    }

    /**
     * @throws Exception
     */
    private function insertCategory(array $category)
    {
        $categoryData = [
            "cat_name" => $category["title"],
            "category_description" => $category["description"],
            "category_parent" => $category["parent"]
        ];

        if (!is_null($targetCategoryId = $this->checkWpmlTr($category["id"], $category["targetLang"]))) {
            $categoryData["cat_ID"] = $targetCategoryId;
        }

        $categoryId = wp_insert_category($categoryData);

        $this->updateWpmlRow($categoryId, (int)$category["id"], $category["targetLang"]);
    }

    /**
     * @throws Exception
     */
    private function updateWpmlRow(int $categoryId, int $sourceCategoryId, string $targetLang)
    {
        try {
            $this->wpdb->update(
                "{$this->wpdb->prefix}icl_translations",
                [
                    "trid" => $this->fetchWpmlTr($sourceCategoryId),
                    "language_code" => $targetLang
                ],
                [
                    "element_id" => $categoryId,
                    "element_type" => $this->elementType
                ]
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function fetchWpmlTr(int $categoryId)
    {
        $query = "SELECT trid FROM {$this->wpdb->prefix}icl_translations WHERE element_type = %s AND language_code = %s AND element_id = %d";
        $args = [$this->elementType, $this->requestBody()->sourceLang, $categoryId];
        return $this->wpdb->get_col(
            $this->wpdb->prepare($query, $args)
        )[0];
    }

    private function checkWpmlTr(int $sourceCategoryId, string $targetLang)
    {
        $query = "SELECT element_id FROM {$this->wpdb->prefix}icl_translations WHERE element_type = %s AND language_code = %s AND trid = %d";
        $args = [$this->elementType, $targetLang, $this->fetchWpmlTr($sourceCategoryId)];
        return $this->wpdb->get_col(
                $this->wpdb->prepare($query, $args)
            )[0] ?? NULL;
    }
}