<?php

class Smartcart_Source_Posts_Handler
{

    private $wpdb;
    private $disableSercretVerificaton = false;

    private $helpers;
    private $log;
    private $request;

    public function __construct()
    {
        $this->helpers = new Smartcart_Helpers;
        $this->log = new Smartcart_Logs();

        global $wpdb;
        $this->wpdb = $wpdb;
    }

    /**
     * @throws ErrorException
     * @throws Exception
     */
    public function request($request)
    {
        $this->request = $request;
        $this->validateRequest();

        return [
            "data" => [
                "status" => "success",
                "posts" => $this->fetchSourcePosts()
            ]
        ];
    }

    /**
     * @throws ErrorException
     */
    private function validateRequest()
    {
        if (!function_exists('icl_object_id')) {
            throw new ErrorException("WPML plugin is not installed or activated. SmartCat Connector is not active", 404);
        }

        if (!$this->helpers->verifySecretKey($this->request) && $this->disableSercretVerificaton === false) {
            throw new ErrorException("Error while authorizing a request by secret key", 401);
        }

        if (is_null($this->params()->sourceLang)) {
            throw new ErrorException("Source lang not found", 500);
        }
    }

    private function params()
    {
        $requestData = $this->request->get_params();
        return (object)[
            "sourceLang" => $requestData["sourceLang"] ?? NULL,
        ];
    }

    private function fetchSourcePosts()
    {
        $query = "SELECT * FROM {$this->wpdb->prefix}icl_translations WHERE language_code = %s AND (element_type = %s OR element_type = %s)";
        $args = [$this->params()->sourceLang, "post_post", "post_page"];
        $result = (array)$this->wpdb->get_results(
            $this->wpdb->prepare($query, $args)
        );
        return $this->normalize($result);
    }

    private function normalize($rows): array
    {
        $posts = [];
        foreach ($rows as $row) {
            if ($this->isTrash($row->element_id)) {
                continue;
            }
            $posts[] = [
                "id" => $row->element_id,
                "type" => explode("_", $row->element_type)[1],
                "lang" => $row->language_code
            ];
        }
        return $posts;
    }

    private function isTrash($postId)
    {
        $post = get_post($postId);
        return (is_null($post) || $post->post_status === "trash");
    }

}