<?php

class Smartcat_Integration_Hub_Public
{

    private $plugin_name;
    private $version;

    private $api_prefix = 'smartcat';

    private $log;

    private $helpers;
    private $pull;
    private $push;
    private $categories;
    private $import;
    private $getPosts;
    private $pushCategories;
    private $languages;
    private $sourcePosts;
    private $status;
    private $allPosts;
    private $metadata;

    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->log = new Smartcart_Logs();
        $this->helpers = new Smartcart_Helpers;
        $this->pull = new Smartcart_Pull;
        $this->categories = new Smartcart_Pull_Categories;
        $this->push = new Smartcart_Push;
        $this->pushCategories = new Smartcart_Push_Categories;
        $this->import = new Smartcart_Import;
        $this->languages = new Smartcart_Languages;
        $this->getPosts = new Smartcart_Get_Posts;
        $this->sourcePosts = new Smartcart_Source_Posts_Handler;
        $this->status = new Smartcart_Status();
        $this->allPosts = new Smartcart_All_Posts();
        $this->metadata = new Smartcart_Get_Metadata();
    }

    /**
     * Регистрация роутов REST API
     */

    public function routes()
    {
        if (!function_exists('wp_parse_list')) {
            function wp_parse_list( $list ) {
                if ( ! is_array( $list ) ) {
                    return preg_split( '/[\s,]+/', $list, -1, PREG_SPLIT_NO_EMPTY );
                }

                return $list;
            }
        }

        if (!function_exists('rest_is_integer')) {
            function rest_is_integer( $maybe_integer ) {
                return is_numeric( $maybe_integer ) && round( (float) $maybe_integer ) === (float) $maybe_integer;
            }
        }

        if (!function_exists('rest_is_array')) {
            function rest_is_array( $maybe_array ) {
                if ( is_scalar( $maybe_array ) ) {
                    $maybe_array = wp_parse_list( $maybe_array );
                }

                return wp_is_numeric_array( $maybe_array );
            }
        }

        register_rest_route($this->api_prefix, "/export/posts", [
            "methods" => "POST",
            "callback" => [$this, "exportPosts"],
            "args" => [
                "date" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return is_string($param) || is_null($param);
                    }
                ],
                "sourceLang" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return is_string($param);
                    }
                ],
                "documentId" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return rest_is_integer($param) || is_null($param);
                    }
                ],
            ],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/export/categories", [
            "methods" => "POST",
            "callback" => [$this, "exportCategories"],
            "args" => [
                "sourceLang" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return !empty($param) && is_string($param);
                    }
                ],
            ],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/push", [
            "methods" => "POST",
            "callback" => [$this, "push"],
            "args" => [
                "documents" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return !empty($param) && is_array($param);
                    }
                ],
            ],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/push/categories", [
            "methods" => "POST",
            "callback" => [$this, "pushCategories"],
            "args" => [
                "categories" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return !empty($param) && is_array($param);
                    }
                ],
            ],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/import", [
            "methods" => "POST",
            "callback" => [$this, "import"],
            "args" => [
                "targetLanguages" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return !empty($param) && rest_is_array($param);
                    }
                ],
                "sourceLang" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return !empty($param) && is_string($param);
                    }
                ],
                "documentId" => [
                    "validate_callback" => function ($param, $request, $key) {
                        return rest_is_integer($param) || is_null($param);
                    }
                ],
            ],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/posts", [
            "methods" => "GET",
            "callback" => [$this, "posts"],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/languages", [
            "methods" => "POST",
            "callback" => [$this, "languages"],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/all-posts", [
            "methods" => "POST",
            "callback" => [$this, "allPosts"],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/source-posts", [
            "methods" => "POST",
            "callback" => [$this, "sourcePosts"],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/status", [
            "methods" => "POST",
            "callback" => [$this, "status"],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/logs", [
            "methods" => "GET",
            "callback" => [$this, "logs"],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/posts/where", [
            "methods" => "GET",
            "callback" => [$this, "postWhere"],
            'permission_callback' => '__return_true'
        ]);
        register_rest_route($this->api_prefix, "/metadata", [
            "methods" => "GET",
            "callback" => [$this, "metadata"],
            'permission_callback' => '__return_true'
        ]);
    }

    public function postWhere($request)
    {
        try {
            return $this->getPosts->where();
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    public function logs($request)
    {
        try {
            return $this->log->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    public function status($request)
    {
        try {
            return $this->status->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    public function sourcePosts($request)
    {
        try {
            return $this->sourcePosts->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    /**
     * Выборка всех постов и страниц для интеграционного теста
     *
     * @return array|array[]
     */

    public function posts()
    {
        try {
            return $this->getPosts->request();
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    /**
     * Все активные языки WPML
     *
     * @param $request
     * @return array|array[]
     */

    public function languages($request)
    {
        try {
            return $this->languages->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    public function allPosts($request)
    {
        try {
            return $this->allPosts->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    /**
     * Выгрузка постов и страниц для команды pull
     *
     * @param $request
     * @return array|array[]
     */

    public function exportPosts($request)
    {
        try {
            return $this->pull->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    /**
     * Выгрузка категорий для команды pull
     *
     * @param $request
     * @return array|array[]
     */

    public function exportCategories($request)
    {
        try {
            return $this->categories->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    /**
     * Создание/Обновление готовых переводов постов и страниц из Smartcat
     *
     * @param $request
     * @return array|string[][]
     */

    public function push($request)
    {
        try {
            return $this->push->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    /**
     * Создание/Обновление готовых переводов категорий из Smartcat
     *
     * @param $request
     * @return array|string[][]
     */

    public function pushCategories($request)
    {
        try {
            return $this->pushCategories->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    /**
     * Готовые переводы для команды import
     *
     * @param $request
     * @return array|array[]
     */

    public function import($request)
    {
        try {
            return $this->import->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

    public function metadata($request)
    {
        try {
            return $this->metadata->request($request);
        } catch (\Exception $e) {
            $this->log->add("Error", $e->getMessage(), $e->getTraceAsString());
            return [
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
                "data" => [
                    "status" => "failed"
                ]
            ];
        }
    }

}
